package com.yixin.ms.server;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.crypto.util.SzhzSm4Util;
import com.yixin.ms.model.enums.HealthCodeLevelEnum;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.dto.GzhHealthDTO;
import com.yixin.ms.server.dto.NucleicTestDTO;
import com.yixin.ms.server.dto.VaccinateDTO;
import com.yixin.ms.server.vo.GzhHealthVO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.IrsTripResultVO;
import lombok.extern.slf4j.Slf4j;
import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.RequestBody;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import java.util.List;
import java.util.Map;

/**
 * @author: lileilei
 * @date: 2022/8/17 14:19
 * @description: 贵州健康码服务
 */
@Slf4j
@Component
public class GuizhouHealthServer {
    @Value("${health.guizhou.gzhHealthByIdCardUrl}")
    private String gzhHealthByIdCardUrl;

    //私钥
    @Value("${health.guizhou.secretKey}")
    private String secretKey;

    @Value("${health.guizhou.identity}")
    private String identity;

    /**
     * 通过身份证查询健康码信息
     *
     * @param dto
     * @return
     */
    public HealtCodeUserVO gzhHealthByIdCard(GzhHealthDTO dto) {
        HealtCodeUserVO userVO = null;
        Map<String, String> headers = MapUtil.newHashMap();
        String requestTime = String.valueOf(System.currentTimeMillis());
        String sign = SecureUtil.md5(new StringBuilder().append(identity).append(secretKey).append(requestTime).toString()).toLowerCase();
        headers.put("sign", sign);
        headers.put("identity", identity);
        headers.put("timestamp", requestTime);
        Map<String, Object> paramMap = MapUtil.newHashMap();
        paramMap.put("cardId", dto.getCardId());
        String body = HttpRequest.post(gzhHealthByIdCardUrl).body(JSONObject.toJSONString(paramMap)).addHeaders(headers).timeout(5000).execute().body();
        log.info("通过身份证查询健康码信息响应内容: {}", body);
        HealthResultVO resultVO = JSON.parseObject(body, HealthResultVO.class);
        if (resultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
            GzhHealthVO gzhHealthVO = JSON.parseObject(JSON.toJSONString(resultVO.getData()), GzhHealthVO.class);
            resultVO.setData(gzhHealthVO);
            userVO = new HealtCodeUserVO();
            List<VaccinateDTO> vaccinate = gzhHealthVO.getVaccinate();
            if (CollectionUtil.isNotEmpty(vaccinate)) {
                userVO.setUsername(vaccinate.stream().findFirst().get().getChil_name());
                userVO.setVaccineNum(String.valueOf(vaccinate.size()));
            }

            userVO.setIdNumber(gzhHealthVO.getPhone());
            userVO.setData(Lists.newArrayList(new HealtCodeTypeVO().setMzt(HealthCodeLevelEnum.codeOf(gzhHealthVO.getColor()).getDesc())));

            if (CollectionUtil.isNotEmpty(gzhHealthVO.getNucleicTest())) {
                NucleicTestDTO nucleicTestDTO = gzhHealthVO.getNucleicTest().stream().findFirst().get();
                if (!nucleicTestDTO.getHsjcjg().equals("检测中")) {
                    userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setChecktime(DateUtil.parseDateTime(nucleicTestDTO.getHsjcsj())).setResult(nucleicTestDTO.getHsjcjg())));
                } else {
                    userVO.setNuclein(Lists.newArrayList());
                }
            } else {
                userVO.setNuclein(Lists.newArrayList());
            }
        }

        return userVO;
    }

    public static void main(String[] args) {
        String requestTime = String.valueOf(System.currentTimeMillis());
        String sign = SecureUtil.md5(new StringBuilder().append("HHGZJKM20221012").append("123456").append(requestTime).toString()).toLowerCase();
        System.out.println("---->"+requestTime);
        System.out.println("---->"+sign);

    }

}
