package com.yixin.ms.server;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.ColorEnum;
import com.yixin.ms.model.enums.HealthOpenEnum;
import com.yixin.ms.model.enums.NucleicStatusEnum;
import com.yixin.ms.model.enums.TravelValidationEnum;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.dto.TokenDataDTO;
import com.yixin.ms.server.vo.HealthResultVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * @ClassName UserBlackServer
 * @Author WXG
 * @Date 2022/9/27 17:17
 */
@Component
@Slf4j
public class UserBlackServer {

    @Value("${health.code.checkUserBlackListURL}")
    private String checkUserBlackListURL;
    @Autowired
    private RestTemplate restTemplate;
    public Boolean checkUserBlackListURL(ShortTermDTO shortTermDTO) {
        Boolean result = null;
        try{
            result = restTemplate.postForObject(checkUserBlackListURL,shortTermDTO,Boolean.class);
        }catch (Exception e){
            log.info("请求失败",e);
            return true;
        }
        log.info("获取黑名单响应内容：{}", result);
        return result != null ? result : true;
    }

    public HealtCodeUserVO basicsResult(ShortTermDTO shortTermDTO) {
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
        userVO.setOpenMsg("未授权，请联系管理员!");
        userVO.setOpenVoiceMsg("未授权，请联系管理员!");
        userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : shortTermDTO.getName());
        userVO.setIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : shortTermDTO.getIdNumber());
        userVO.setBackgroundColor(ColorEnum.WHITE.getValue());
        userVO.setVaccineNumColor(ColorEnum.WHITE.getValue());
        userVO.setVaccineIconUrl(NucleicStatusEnum.NOT_AVAILABLE.getUrl());
        userVO.setTravelValidationIconUrl(TravelValidationEnum.HAVE_BEEN_TO.getUrl());
        userVO.setTitle("禁止通行");
        HealtCodeNucleinVO healtCodeNucleinVO = new HealtCodeNucleinVO();
        healtCodeNucleinVO.setContent("无");
        healtCodeNucleinVO.setTitle("核酸检测");
        List<HealtCodeNucleinVO> list = new ArrayList<>();
        list.add(healtCodeNucleinVO);
        userVO.setNuclein(list);

        HealtCodeTypeVO healtCodeTypeVO = new HealtCodeTypeVO();
        healtCodeTypeVO.setContent("未授权，请联系管理员!");
        List<HealtCodeTypeVO> healtCodeTypeVOList = new ArrayList<>();
        healtCodeTypeVOList.add(healtCodeTypeVO);
        userVO.setData(healtCodeTypeVOList);

        userVO.setVaccineIcoTitle(StrUtil.EMPTY);
        userVO.setTravelValidationTitle(StrUtil.EMPTY);
        userVO.setTravelValidationDesc(StrUtil.EMPTY);
        return userVO;
    }
}
